using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Microsoft.HomeServer.Extensibility;
using Microsoft.HomeServer.Common.Client;
using Microsoft.HomeServer.Controls;

namespace Microsoft.HomeServer.HomeServerConsoleTab.FancyListViewDemoCS
{
   public partial class MainTabUserControl : UserControl
   {
      IConsoleServices consoleServices;

      public MainTabUserControl()
      {
         InitializeComponent();
      }

      public MainTabUserControl(int width, int height, IConsoleServices consoleServices)
         : this()
      {
         this.Width = width;
         this.Height = height;
         this.consoleServices = consoleServices;
      }

      System.Timers.Timer timer1;
      private void MainTabUserControl_Load(object sender, EventArgs e)
      {
         timer1 = new System.Timers.Timer(100);
         timer1.Elapsed += new System.Timers.ElapsedEventHandler(timer1_Elapsed);
         
         ImageList imageList = new ImageList();
         imageList.ImageSize = new Size(32, 32);
         imageList.ColorDepth = ColorDepth.Depth32Bit;

         //Add folder icons to image list
         imageList.Images.Add("Music", CommonImages.MusicIcon32x32);
         imageList.Images.Add("Videos", CommonImages.VideosIcon32x32);
         imageList.Images.Add("Public", CommonImages.PublicIcon32x32);
         imageList.Images.Add("Photos", CommonImages.PhotosIcon32x32);
         imageList.Images.Add("Folder", CommonImages.SharedFolderIcon32x32);

         //Add led icons to image list
         imageList.Images.Add("GrayIcon", CommonImages.GrayIcon32);
         imageList.Images.Add("GreenIcon", CommonImages.GreenIcon32);
         imageList.Images.Add("YellowIcon", CommonImages.YellowIcon32);
         imageList.Images.Add("RedIcon", CommonImages.RedIcon32);

         //Add duplication icons to image list
         imageList.Images.Add("DuplicationDisabled", CommonImages.DupDisabledIcon);
         imageList.Images.Add("DuplicationEnabled", CommonImages.DupEnabledIcon);

         fancyListView1.SmallImageList = imageList;

         ListViewItem lvi;
         Random r = new Random();

         for (int x = 0; x < 10; x++)
         {
            lvi = new ListViewItem();
            lvi.Text = "Folder #" + x.ToString();
            //Randomly select a folder icon from the imagelist
            lvi.ImageIndex = r.Next(0, 5);

            //Status icon
            FancyListView.ImageSubItem isi = new FancyListView.ImageSubItem();
            //Randomly select a status icon from the imagelist
            isi.Text = imageList.Images.Keys[r.Next(5, 9)];
            isi.ImageKey = isi.Text;
            
            //Add status icon to ListViewItem
            lvi.SubItems.Add(isi);


            //Duplication icon
            FancyListView.ImageSubItem isi1 = new FancyListView.ImageSubItem();
            if (r.Next(0, 2) == 0)
            {
               isi1.ImageKey = "DuplicationDisabled";
            }
            else
            {
               isi1.ImageKey = "DuplicationEnabled";
            }

            //Add duplication icon to ListViewItem
            lvi.SubItems.Add(isi1);         

            //Add new ListViewItem to FancyListView
            fancyListView1.Items.Add(lvi);
         }
         timer1.Start();
      }

      ListViewItem currentItem = null;
      void timer1_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
      {
         //Required to make sure that update is happening on proper thread
         if (this.InvokeRequired)
         {
            this.Invoke(new System.Timers.ElapsedEventHandler(timer1_Elapsed), new object[] { sender, e });
            return;
         }

         if (this.Parent == null)
            return;

         if (currentItem == null)
         {
            currentItem = fancyListView1.Items[0];
         }
         
         FancyListView.FilledBarSubItem fbsi;
         
         //if no FilledBarSubItem exists, create one
         if (currentItem.SubItems.Count == 3)
         {
            fbsi = new FancyListView.FilledBarSubItem();

            //Randomly select warning and critical thresholds
            Random r = new Random();
            int warning = r.Next(0, 100);
            int critical = r.Next(warning, 100);

            fbsi.WarningPercentage = warning;
            fbsi.CriticalPercentage = critical;

            fbsi.WarningColor = Color.Orange;
            fbsi.CriticalColor = Color.Red;

            currentItem.SubItems.Add(fbsi);
         }
         else
         {
            fbsi = currentItem.SubItems[3] as FancyListView.FilledBarSubItem;
         }

         
         if (fbsi.PercentageFilled == 100)
         {
            //Remove current FilledBarSubItem
            currentItem.SubItems.Remove(fbsi);

            //Find location of current item and increment
            int newLocation = fancyListView1.Items.IndexOf(currentItem) + 1;

            //If new item index is higher than avalible items, wrap
            if (newLocation >= fancyListView1.Items.Count)
            {
               newLocation = 0;
            }
            //Specify next/currentItem
            currentItem = fancyListView1.Items[newLocation];

            //Recall this method with new currentItem
            timer1_Elapsed(sender, e);
            return;
         }

         //Increment current percentage
         fbsi.PercentageFilled++;
         fbsi.Text = fbsi.PercentageFilled + "%";

         //Locate index of currentItem
         int index = fancyListView1.Items.IndexOf(currentItem);
         //Redraw just the currentItem
         fancyListView1.RedrawItems(index, index, false);
      }
   }
}
